/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>



#define VER         "0.1"
#define BUFFSZ      8192

#define READ_DATA   fread(&len, 4, 1, fd); \
                    if(len > BUFFSZ) exit(1); \
                    p = buff; \
                    while(len--) { \
                        *p++ = fgetc(fd); \
                        fgetc(fd); \
                    } \
                    *p = 0;



void lotr3_pwd(unsigned char *pwd);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *fd;
    int             len;
    unsigned char   buff[BUFFSZ + 1],
                    *p;


    setbuf(stdout, NULL);

    fputs("\n"
        "Lords of the Realm III account password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <gamespyinfo.dat>\n"
            "\n", argv[0]);
        fputs("\n   Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("- open file \"%s\"\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    fseek(fd, 8, SEEK_SET);

    READ_DATA;
    printf("\n  Username:   %s\n", buff);

    READ_DATA;
    printf("  E-mail:     %s\n", buff);

    if(fgetc(fd)) {
        READ_DATA;
        lotr3_pwd(buff);
        printf("  Password:   %s\n", buff);
    }

    fclose(fd);
    fputs("\n   Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void lotr3_pwd(unsigned char *pwd) {
    const  static char      encdata[] = "oqwieuasdfkjhzxv";
    static unsigned char    enc[256];
    unsigned char           p,
                            tmp;
    int                     i;

    for(i = 0; i < 256; i++) {
        enc[i] = i;
    }

    for(i = tmp = 0; i < 256; i++) {
        p = enc[i];
        tmp += p + encdata[i % 16];
        enc[i] = enc[tmp];
        enc[tmp] = p;
    }

    for(i = tmp = 0; *pwd; pwd++) {
        p = enc[++i];
        tmp += p;
        enc[i] = enc[tmp];
        enc[tmp] = p;
        *pwd ^= enc[(p + enc[i]) & 0xff];
    }
}



void std_err(void) {
    perror("\nError");
    fputs("\n   Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


